<?php

/**
 * @file
 * Function for preparing to create a payment with GoCardless.
 */

/**
 * Calculates an amount in preparation for creating a payment.
 *
 * @param object $order
 *   The Commerce order the payment is for.
 * @param object $line_item_wrapper
 *   The Commerce line item wrapper the payment is for.
 * @param string $op
 *   The operation that is creating the payment, either during checkout,
 *   or a scheduled payment creation during a cron run.
 *
 * @return array
 *   An array of values relating to a payment, including the amount to be
 *   created and the currency code.
 *
 * @see commerce_gc_client_cron()
 * @see commerce_gc_client_redirect_form_submit()
 */
function commerce_gc_client_price_calculate($order, $line_item_wrapper, $op = NULL, $next_payment = NULL) {

  $line_item = $line_item_wrapper->value();

  // Process any scheduled adjustments associated with the payment.
  $adjustments_total = 0;
  if ($op != 'new') {
    $adjustments_array = array();
    $query = db_select('commerce_gc_client_item', 'i');
    $query->join('commerce_gc_client_item_schedule', 's', 'i.line_item_id = s.line_item_id');
    $adjustments = $query->fields('s', array('sid', 'data', 'date', 'status'))
      ->condition('i.line_item_id', $line_item->line_item_id)
      ->condition('s.status', 1)
      ->condition('s.date', $next_payment)
      ->condition('s.type', 'adj')
      ->orderBy('timestamp', 'ASC')
      ->execute()
      ->fetchAll();

    foreach ($adjustments as $adjustment) {
      $adjustment_data = unserialize($adjustment->data);
      $adjustments_array[] = array($adjustment_data['title'] => $adjustment_data['amount']);
      $adjustments_total = $adjustments_total + $adjustment_data['amount'];
    }
  }

  // Include additional line item costs associated with this one
  // (i.e. shipping, tax, discounts)
  $calculate_total = 0;
  $calculate_array = array();
  if (isset($line_item->data['gc']['calculate'])) {
    foreach ($line_item->data['gc']['calculate'] as $calculate_key => $calculate_amount) {
      $calculate_array[$calculate_key] = $calculate_amount;
      $calculate_total = $calculate_total + $calculate_amount;
    }
  }

  $line_item_amount = $line_item_wrapper->commerce_total->value()['amount'];
  $amount = $line_item_amount + $adjustments_total + $calculate_total;

  // Modify payment amount and currency if international customer.
  $scheme = db_select('commerce_gc_client', 'g')
    ->fields('g', array('gc_mandate_scheme'))
    ->condition('order_id', $order->order_number)
    ->execute()->fetchField();

  foreach (variable_get('commerce_gc_client_currency_schemes') as $gc_currency_code => $gc_currency) {
    if ($gc_currency['scheme'] == $scheme) {
      break;
    }
  }

  $settings = commerce_gc_client_load_settings();

  if ($settings['global']['currencies']) {

    if ($fixer = $settings['global']['fixer']) {
      $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
      $line_item_currency_code = $order_wrapper->commerce_order_total->value()['currency_code'];

      if ($gc_currency_code != $line_item_currency_code) {

        // Use fixer.io to provide latest exchange rates for currency
        // conversion.
        $uri = 'http://data.fixer.io/api/latest?access_key=' . $fixer;

        $options = array(
          'headers' => array('Content-Type' => 'application/json',
          ),
          'method' => 'GET',
        );
        $result = drupal_http_request($uri, $options);

        if (isset($result->error)) {
          drupal_set_message(t('We were unable to obtain a currency exchange rate at this time'), 'warning');
        }
        else {
          $data = json_decode($result->data);
          if ($line_item_currency_code == 'EUR') {
            $exchange_rate = $data->rates->$line_item_currency_code;
            $amount = $amount * $exchange_rate;
          }
          elseif ($gc_currency_code == 'EUR') {
            $exchange_rate = $data->rates->$line_item_currency_code;
            $amount = $amount / $exchange_rate;
          }
          else {
            $exchange_rate_1 = $data->rates->$line_item_currency_code;
            $exchange_rate_2 = $data->rates->$gc_currency_code;
            $amount = ($amount / $exchange_rate_1) * $exchange_rate_2;
          }

          if ($op == 'new') {
            $symbol = commerce_currency_get_symbol($gc_currency_code);
            drupal_set_message(t('The amount for your purchase has been converted into @symbol using current exchange rates at fixer.io', array(
              '@symbol' => $symbol,
            )));
          }
        }
      }
    }
  }
  return array(
    'description' => 'Payment for Order #' . $order->order_number,
    'line_item_amount' => $line_item_amount,
    'adjustments' => isset($adjustments_array) ? $adjustments_array : NULL,
    'adjustments_total' => $adjustments_total,
    'calculate' => $calculate_array,
    'calculate_total' => $calculate_total,
    'amount' => round($amount),
    'exchange_rate' => isset($exchange_rate) ? $exchange_rate : NULL,
    'currency_code' => $gc_currency_code,
  );
}
