<?php

/**
 * @file
 * Functions for handling multiple currencies with GoCardless.
 */

/**
 * Form constructor for the GoCardless Currencies form.
 *
 * @see commerce_gc_client_currencies_form_submit();
 */
function commerce_gc_client_currencies_form($form, &$form_state) {
  $currencies = variable_get('commerce_gc_client_currency_schemes');
  $commerce_currencies = commerce_currencies(TRUE);
  include_once DRUPAL_ROOT . '/includes/locale.inc';
  $countries = country_get_list();
  $rows = array();
  $rows_unavailable = array();
  $defaults = array();
  foreach ($currencies as $currency_code => & $currency) {

    $currency_countries = array();
    foreach ($currency['countries'] as $country_code => $country) {

      $country_name = $countries[$country_code];
      if (isset($country['failure_warning'])) {
        $country_name .= '*';
      }
      $currency_countries[$country_code] = $country_name;
    }
    $currency_countries = implode(', ', $currency_countries);

    $row = array(
      $currency_code,
      $currency['scheme'],
      $currency_countries,
    );

    if (isset($commerce_currencies[$currency_code])) {
      $rows[$currency_code] = $row;
      if ($currency['enabled']) {
        $defaults[$currency_code] = TRUE;
      }
    }
    else {
      $rows_unavailable[$currency_code] = $row;
    }
  }
  $header = array(t('Currency code'), t('Scheme'), t('Country / countries'));

  $markup = '<p>' . t('For information on currencies and countries that are serviced by GoCardless <a target="new" href="@url1">view their FAQ page here</a>. To collect payments from multiple regions the relevant direct debit schemes need to be enabled by GoCardless. See <a target="new" href="@url2">GoCardless article</a> for more information on how to do this.', array(
    '@url1' => 'https://gocardless.com/faq/merchants/international-payments',
    '@url2' => 'https://support.gocardless.com/hc/en-gb/articles/115002833785-Collecting-from-multiple-regions',
  )) . '</p>';

  $markup .= '<p>' . t('Before a GoCardless currency can be used, it  must be enabled in "Currency Settings". Also make sure that the relevant countries are enabled in the <a href="@path">Customer Billing Profile</a>.', array(
    '@path' => '/admin/commerce/customer-profiles/types/billing/fields/commerce_customer_address',
  )) . '</p>';

  $form['markup'] = array(
    '#markup' => $markup,
  );

  $form['currencies'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $rows,
    '#empty' => t('No countries available.'),
    '#default_value' => $defaults,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Enable / disable currencies'),
  );

  $form['markup_unavailable'] = array(
    '#markup' => '<p>' . t('GoCardless currencies that are currently unavailable:') . '</p>',
  );

  $form['currencies_unavailable'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows_unavailable,
    '#empty' => t('All currencies available.'),
  );

  $form['markup_failure_warning'] = array(
    '#markup' => '<p style = "font-size: smaller;">' . t('* Direct Debits in these countries have a relatively high failure rate.') . '</p>',
  );
  return $form;
}

/**
 * Form submission handler for commerce_gc_client_currencies_form().
 */
function commerce_gc_client_currencies_form_submit($form, &$form_state) {
  $currencies = variable_get('commerce_gc_client_currency_schemes');
  $values = $form_state['values']['currencies'];
  foreach ($currencies as $currency_code => $currency) {
    $enabled = FALSE;
    if (isset($values[$currency_code])) {
      $values[$currency_code] ? $enabled = TRUE : NULL;
    }
    $currencies[$currency_code]['enabled'] = $enabled;
  }
  variable_set('commerce_gc_client_currency_schemes', $currencies);
  drupal_set_message(t('Available GoCardless currencies have been updated.'));
}

/**
 * Returns an array of currencies and their associated direct debit schemes.
 *
 * @return array
 *   A multi-dimensional array keyed at the top level by three digit currency
 *   codes, with values holding information on currencies and their
 *   correxponding direct debit schemes.
 *
 * @see commerce_gc_client.install()
 */
function commerce_gc_client_currency_schemes() {
  return array(
    'GBP' => array(
      'scheme' => 'bacs',
      'countries' => array(
        'GB' => array(),
      ),
    ),
    'SEK' => array(
      'scheme' => 'autogiro',
      'countries' => array(
        'SE' => array(),
      ),
    ),
    'AUD' => array(
      'scheme' => 'becs',
      'countries' => array(
        'AU' => array(),
      ),
    ),
    'DKK' => array(
      'scheme' => 'betalingsservice',
      'countries' => array(
        'DK' => array(),
      ),
    ),
    'NZD' => array(
      'scheme' => 'becs_nz',
      'countries' => array(
        'NZ' => array(),
      ),
    ),
    'CAD' => array(
      'scheme' => 'pad',
      'countries' => array(
        'CA' => array(),
      ),
    ),
    'EUR' => array(
      'scheme' => 'sepa_core',
      'countries' => array(
        'AX' => array(
          'unsupported' => 1,
        ),
        'AT' => array(),
        'BE' => array(),
        'CY' => array(),
        'EE' => array(),
        'FI' => array(
          'failure_warning' => 1,
        ),
        'FR' => array(),
        'GF' => array(
          'unsupported' => 1,
        ),
        'DE' => array(),
        'GR' => array(
          'failure_warning' => 1,
        ),
        'GP' => array(
          'unsupported' => 1,
        ),
        'IE' => array(),
        'IT' => array(),
        'LV' => array(
          'unsupported' => 1,
        ),
        'LT' => array(
          'failure_warning' => 1,
        ),
        'LU' => array(),
        'MT' => array(),
        'MQ' => array(
          'unsupported' => 1,
        ),
        'YT' => array(
          'unsupported' => 1,
        ),
        'MC' => array(),
        'NL' => array(),
        'PT' => array(),
        'RE' => array(
          'unsupported' => 1,
        ),
        'BL' => array(
          'unsupported' => 1,
        ),
        'MF' => array(
          'unsupported' => 1,
        ),
        'PM' => array(
          'unsupported' => 1,
        ),
        'SM' => array(
          'unsupported' => 1,
        ),
        'SK' => array(
          'failure_warning' => 1,
        ),
        'SI' => array(),
        'ES' => array(),
      ),
    ),
    'USD' => array(
      'scheme' => 'ach',
      'countries' => array(
        'US' => array(),
      ),
    ),
  );
}
