<?php

/**
 * @file
 * Provides an interface for managing customer's GoCardless mandates.
 */

/**
 * Form constructor for the GoCardless Mandates form.
 *
 * @param object $order
 *   The Commerce Order.
 *
 * @see commerce_gc_client_subscription_update_validate()
 * @see commerce_gc_client_subscription_update_submit()
 * @see commerce_gc_client_subscription_cancel_submit()
 * @see commerce_gc_client_adjust_validate()
 * @see commerce_gc_client_adjust_submit()
 * @see commerce_gc_client_create_payment_validate()
 * @see commerce_gc_client_create_payment_submit()
 * @see commerce_gc_client_update_payment_submit()
 * @see commerce_gc_client_payment_cancel()
 * @see commerce_gc_client_payment_cancel_submit()
 * @see commerce_gc_client_mandates_form_callback()
 * @see commerce_gc_client_scheduled_action()
 * @see commerce_gc_client_scheduled_action_submit()
 */
function commerce_gc_client_mandates_form($form, &$form_state, $order) {
  // Query database for GoCardless mandates associated with order.
  $query = db_select('commerce_gc_client', 'g');
  $query->join('commerce_gc_client_item', 'i', 'i.gcid=g.gcid');
  $gcs = $query->fields('g')->fields('i')->condition('order_id', $order->order_id)->execute()->fetchAllAssoc('line_item_id');
  $count = count($gcs);

  if ($count == 0) {
    drupal_set_message(t('You have no GoCardless mandates associated with this order.'), 'warning');
    return;
  }

  // Continue only if there are mandate(s) for the order.
  drupal_add_css(drupal_get_path('module', 'commerce_gc_client') . '/commerce_gc_client.css');
  global $user;
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  $commerce_currency_code = $order_wrapper->commerce_order_total->value()['currency_code'];

  // Obtain the currency code for the mandate from the 'scheme' value stored
  // in database.
  foreach (variable_get('commerce_gc_client_currency_schemes') as $currency_code => $gc_currency) {
    if ($gc_currency['scheme'] == reset($gcs)->gc_mandate_scheme) {
      break;
    }
  }

  // Loop through the order's line items and create an array of unique
  // line item names, based on the line item ID and title.
  $line_item_select = array();
  $title_count = array();
  foreach ($order_wrapper->commerce_line_items as $line_item_wrapper) {

    if ($line_item_wrapper->type->value() != 'product') {
      continue;
    }
    $select_id = $line_item_wrapper->value()->line_item_id;
    $line_item_title = $line_item_wrapper->commerce_product->value()->title;
    !isset($title_count[$line_item_title]) ? $title_count[$line_item_title] = 1 : $title_count[$line_item_title]++;
    $title_count[$line_item_title] == 1 ? $title[$select_id] = $line_item_title : $title[$select_id] = $line_item_title . ' (' . $title_count[$line_item_title] . ')';
    $line_item_select[$select_id] = $title[$select_id];
  }

  // If there is only one line item in the order then create the title
  // section of the form.
  if ($count == 1) {
    $line_item_id = reset($gcs)->line_item_id;
    $text_top = '<h3><b>' . t('Administrate <a href = "@url">@title</a>', array(
      '@title' => isset($line_item_title) ? $line_item_title : NULL,
      '@url' => url('admin/commerce/orders/' . $order->order_number),
    )) . '</b></h3>';

    $form['text_top'] = array(
      '#type' => 'item',
      '#markup' => $text_top,
    );
    $form['line_item_id'] = array(
      '#type' => 'value',
      '#value' => $line_item_id,
    );
  }

  // Else if there is more than one line item choose the default line item,
  // create the title section of the form, and add a select list in order
  // to switch between line items.
  else {
    if (isset($_SESSION['commerce_gc_client_line_item_id_' . $user->uid . '_' . $order->order_id])) {
      $line_item_id = $_SESSION['commerce_gc_client_line_item_id_' . $user->uid . '_' . $order->order_id];
    }
    elseif (!isset($line_item_id)) {
      $line_item_id = reset($gcs)->line_item_id;
    }

    $form = array(
      '#prefix' => '<div id="change-line-item">',
      '#suffix' => '</div>',
    );

    $text_top = '<h3><b>' . t('Administrate <a href = "@url">@title</a>', array(
      '@title' => $line_item_select[$line_item_id],
      '@url' => url('admin/store/orders/' . $order->order_number),
    )) . '</b></h3>';

    $form['text_top'] = array(
      '#type' => 'item',
      '#markup' => $text_top,
    );

    $form['line_item_id'] = array(
      '#title' => t('Switch line item'),
      '#type' => 'select',
      '#options' => $line_item_select,
      '#default_value' => $line_item_id,
      '#ajax' => array(
        'callback' => 'commerce_gc_client_mandates_form_callback',
        'wrapper' => 'change-line-item',
      ),
    );
  }

  // Access GoCardless API for mandate details of selected line item.
  $gc = $gcs[$line_item_id];
  $partner = new CommerceGcClientPartner();
  $result = $partner->api([
    'endpoint' => 'mandates',
    'action' => 'get',
    'mandate' => $gc->gc_mandate_id,
  ]);
  if ($result->response->status_code != 200) {
    drupal_set_message(t('No active mandate found for this order at GoCardless.'), 'warning');
    return;
  }
  else {
    $mandate = $result->response->body->mandates;
  }

  // Load the commerce line item wrapper and extract relevant data.
  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item_id);
  $total = $line_item_wrapper->commerce_total->value();
  $line_item = $line_item_wrapper->value();
  isset($line_item->data['gc']) ? $data = $line_item->data['gc'] : $data = NULL;
  $interval = isset($data['interval_params']) ? $data['interval_params']['string'] : NULL;

  // Loop throught the 'calculate' element of the GoCardless data stored in
  // line item object looking for entries relating to shipping, tax, and
  // discounts, and add to an array keyed by the type of line item.
  $calculate_array = array();
  if (isset($data['calculate'])) {
    foreach ($data['calculate'] as $calculate_key => $calculate_data) {
      if (is_numeric($calculate_data)) {
        $calculate_data = commerce_currency_format($calculate_data, $commerce_currency_code);
      }
      $calculate_array[ucfirst($calculate_key)] = $calculate_data;
    }
  }

  // Access GoCardless API for any payments associated with the line item.
  $result = $partner->api([
    'endpoint' => 'payments',
    'action' => 'list',
    'mandate' => $gc->gc_mandate_id,
    'limit' => 500,
  ]
  );
  if ($result->response->status_code == 200) {
    $payments = $result->response->body->payments;
  }

  // If the line item is of 'One-off payments' GoCardless type.
  if ($gc->type == 'P') {

    $default_name = $line_item_select[$line_item_id];
    $default_amount = number_format((float) $total['amount'] / 100, 2, '.', '');

    // Assemble the data for the 'Product Data' table within the form.
    $product_data = array(
      t('Mandate ID') => check_plain($mandate->id),
      t('Mandate status') => check_plain($mandate->status),
      t('Created') => format_date(strtotime($mandate->created_at), 'short'),
      t('Type') => t('Payments'),
      t('Amount') => commerce_currency_format($total['amount'], $commerce_currency_code),
    );
    $product_data = $product_data + $calculate_array + array(
      t('Interval') => $interval == ' ' ? '-' : $interval,
      t('Next possible charge date') => !is_null($mandate->next_possible_charge_date) ? format_date(strtotime($mandate->next_possible_charge_date), 'gocardless_client') : '-',
      t('Next scheduled payment creation') => $mandate->status != 'cancelled' && !is_null($gc->next_payment) ? format_date($gc->next_payment, 'gocardless_client') : '-',
    );

    // Assemble the data for scheduled adjustments table.
    $query = db_select('commerce_gc_client_item', 'i');
    $query->join('commerce_gc_client_item_schedule', 's', 's.line_item_id=i.line_item_id');
    $adjustments = $query->fields('s')->condition('i.line_item_id', $gc->line_item_id)->condition('s.type', 'adj')->orderBy('timestamp', 'ASC')->execute()->fetchAll();

    if (!empty($adjustments)) {

      $adjustment_headers = array(
        t('Title'),
        t('Adjustment'),
        t('Date'),
        t('Status'),
        '',
      );
      $adjustment_rows = array();
      foreach ($adjustments as $adjustment) {

        $adjustment->status == 1 ? $cancel = l(t('Cancel'), "/gc_client/scheduled_action/cancel/" . $gc->order_id . '/' . $adjustment->sid, array()) : $cancel = NULL;
        $adjustment->status == 1 ? $spacer = ' | ' : $spacer = NULL;
        $delete = l(t('Delete'), "/gc_client/scheduled_action/delete/" . $gc->order_id . '/' . $adjustment->sid, array());
        $actions = $cancel . $spacer . $delete;

        if ($adjustment->status == 0) {
          $status = 'Cancelled';
        }
        elseif ($adjustment->status == 1) {
          $status = 'Pending';
        }
        else {
          $status = 'Complete';
        }

        $adjustment_data = unserialize($adjustment->data);
        $adjustment_rows[] = array(
          $adjustment_data['title'],
          commerce_currency_format($adjustment_data['amount'], $currency_code),
          $adjustment->date,
          $status,
          $actions,
        );
      }
    }
  }

  // Else if the line item is of 'Subscription payments' GoCardless type.
  elseif ($gc->type == 'S') {

    // Access GoCardless API for the subscription data for the line item.
    $subscription = array();
    if ($gc->gc_subscription_id) {
      $result = $partner->api([
        'endpoint' => 'subscriptions',
        'action' => 'get',
        'id' => $gc->gc_subscription_id,
      ]
      );
      if ($result->response->status_code == 200) {
        $subscription = $result->response->body->subscriptions;
      }
    }

    if (!count($subscription) || !$gc->gc_subscription_id) {
      drupal_set_message(t('No subscription found at GoCardless for this line item.'), 'warning');
      return $form;
    }

    // Assemble the data for the 'Product Data' table within the form.
    $product_data = array(
      t('Mandate ID') => check_plain($mandate->id),
      t('Mandate status') => check_plain($mandate->status),
      t('Created') => format_date(strtotime($mandate->created_at), 'gocardless_client'),
      t('Type') => t('Subscription'),
      t('Status') => check_plain($subscription->status),
      t('Subscription name') => check_plain($subscription->name),
      t('Subscription ID') => check_plain($subscription->id),
      t('Subscription start') => format_date(strtotime($subscription->start_date), 'gocardless_client'),
      t('Amount') => commerce_currency_format($total['amount'], $commerce_currency_code),
    );
    $product_data = $product_data + $calculate_array + array(
      t('Interval') => $interval,
      t('Day of month') => check_plain($subscription->day_of_month),
      t('Month') => check_plain($subscription->month),
      t('End date') => !is_null($subscription->end_date) ? format_date(strtotime($subscription->end_date), 'gocardless_client') : NULL,
    );

    // Assemble the data for the subscription's upcoming payments table.
    $upcoming_rows = array();
    foreach ($subscription->upcoming_payments as $upcoming) {
      $upcoming_rows[] = array(
        format_date(strtotime($upcoming->charge_date), 'gocardless_client'),
        commerce_currency_format($upcoming->amount, $currency_code),
      );
    }

    $default_name = check_plain($subscription->name);
    $default_amount = number_format((float) $subscription->amount / 100, 2, '.', '');
  }

  // For both One-off and Subsription Payment line items, loop through the
  // assembled product data and create an array of rows for the 'Product Data'
  // table.
  $product_rows = array();
  foreach ($product_data as $product_data_key => $product_data_data) {
    if (is_null($product_data_data)) {
      continue;
    }
    $product_rows[] = array(
      '<b>' . t('@key', array('@key' => $product_data_key)) . '</b>',
      $product_data_data,
    );
  }

  // For both One-off and Subsription Payment line items, loop through the
  // Payments data obtained from GoCardless and create an array of rows for
  // the Payments table in the form.
  $payment_rows = array();
  $payment_total = 0;
  if (isset($payments)) {
    foreach ($payments as & $payment) {

      if (isset($payment->links->subscription) && $payment->links->subscription != $gc->gc_subscription_id || isset($payment->metadata->line_item_id) && $payment->metadata->line_item_id != $line_item_id
      ) {
        continue;
      }

      if ($payment->status == 'pending_submission') {
        $link = l(t('Cancel'), '/gc_client/payment_cancel/' . $order->order_number . '/' . $payment->id);
      }
      else {
        $link = NULL;
      }
      $payment_rows[$payment->id] = array(
        format_date(strtotime($payment->created_at), 'short'),
        check_plain($payment->description),
        check_plain($payment->id),
        commerce_currency_format($payment->amount, $currency_code),
        check_plain($payment->status),
        format_date(strtotime($payment->charge_date), 'gocardless_client'),
        $link,
      );
      if (in_array($payment->status, array('confirmed', 'paid_out'))) {
        $payment_total = $payment_total + $payment->amount;
      }
    }
  }

  // The Product Data section.
  $product_header = array(
    array(
      'data' => t('Product data'),
      'colspan' => 2,
      'style' => 'text-align:center;font-weight:bold;',
    ),
  );

  $form['table'] = array(
    '#theme' => 'table',
    '#header' => $product_header,
    '#rows' => isset($product_rows) ? $product_rows : NULL,
    '#empty' => t('There are no GoCardless products for this order'),
    '#prefix' => '<div id="gc-product-table">',
    '#suffix' => '</div>',
  );

  // The Upcoming Payments section for Subscription payments.
  if ($gc->type == 'S') {
    $form['upcoming_payments'] = array(
      '#type' => 'fieldset',
      '#title' => t('Upcoming payments'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $upcoming_header = array(t('Charge date'), t('Amount'));
    $form['upcoming_payments']['upcoming_payments_table'] = array(
      '#theme' => 'table',
      '#header' => $upcoming_header,
      '#rows' => isset($upcoming_rows) ? $upcoming_rows : NULL,
      '#empty' => t('There are no upcoming payments for this subscription.'),
      '#prefix' => '<div id="gc-upcoming-payments-table">',
      '#suffix' => '</div>',
    );
  }

  // If returning to the form after submitting the 'Create A Payment' section
  // then include a message immediately above the 'Payments' section.
  if (isset($_GET['payment'])) {
    $form['payment_action_message'] = array(
      '#prefix' => '<div style="padding-top:10px;" id="payment"></div>',
      '#type' => 'item',
      '#markup' => $_GET['payment'] == 'cancelled' ? '<div class="messages status">' . t('Payment @payment_id has been @action.', array(
        '@payment_id' => $_GET['payment_id'],
        '@action' => $_GET['payment'],
      )) . '</div>' : NULL,
    );
  }

  // If the GoCardless API call provided data on Payments created for the line
  // item, provide table listing them.
  if (isset($payment_rows)) {

    if (isset($_GET['payment_create'])) {
      $form['payment_create_message'] = array(
        '#prefix' => '<div style="padding-top:10px;" id="payment_create"></div>',
        '#type' => 'item',
        '#markup' => '<div class="messages status">' . t('A one-off payment for @amount has been created with GoCardless.', array(
          '@amount' => commerce_currency_format($_GET['payment_create'], $currency_code),
        )) . '</div>',
      );
    }

    $form['payment_tables'] = array(
      '#type' => 'fieldset',
      '#title' => t('Payments'),
      '#description' => t('(Live data from GoCardless.com)'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    $payment_header = array(
      t('Created'),
      t('Description'),
      t('Payment ID'),
      t('Amount'),
      t('Status'),
      t('Charge customer at'),
      '',
    );
    $form['payment_tables']['payments_table'] = array(
      '#theme' => 'table',
      '#header' => $payment_header,
      '#rows' => isset($payment_rows) ? $payment_rows : NULL,
      '#empty' => t('There are no payments for this product yet.'),
      '#suffix' => t('Total confirmed payments: @payment_total', array('@payment_total' => commerce_currency_format($payment_total, $currency_code))),
    );
  }

  // The 'Create A Payment' section.
  if (in_array($mandate->status, array(
    'pending_submission',
    'submitted',
    'active',
  ))) {

    $form['create_payment'] = array(
      '#type' => 'fieldset',
      '#title' => t('Create a payment'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    if ($gc->type == 'S') {
      $form['create_payment']['#description'] = t('You can create one-off payments under the existing mandate for this product, which are in addition to the subscription plan listed here.');
    }

    $form['create_payment']['payment_amount'] = array(
      '#type' => 'textfield',
      '#title' => t('Amount @symbol', array(
        '@symbol' => commerce_currency_get_symbol($currency_code),
      )),
      '#size' => 10,
      '#default_value' => $default_amount,
    );

    $form['create_payment']['payment_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Payment title'),
      '#default_value' => t('Payment for @default_name', array(
        '@default_name' => $default_name,
      )),
    );

    $form['create_payment']['charge_date'] = array(
      '#title' => t('Charge customer at'),
      '#type' => module_exists('date_popup') ? 'date_popup' : 'date_select',
      '#date_format' => module_exists('date_popup') ? variable_get('date_format_uc_store') : 'd M Y',
      '#default_value' => date('Y-m-d', strtotime($mandate->next_possible_charge_date)),
      '#date_year_range' => '0:+1',
      '#datepicker_options' => array('minDate' => 0),
    );

    $form['create_payment']['payment_submit'] = array(
      '#type' => 'submit',
      '#value' => t('Instruct GoCardless to create a Payment'),
      '#validate' => array('commerce_gc_client_create_payment_validate'),
      '#submit' => array('commerce_gc_client_create_payment_submit'),
      '#attributes' => array('onclick' => 'if (!confirm("Are you sure you want to create a payment with GoCardless?")) {return false;}'),
    );
  }

  if ($gc->type == 'S') {

    if ($subscription->status != 'cancelled') {
      $form['subscription_id'] = array(
        '#type' => 'value',
        '#value' => $gc->gc_subscription_id,
      );

      // Update subscription section.
      $form['update_subscription'] = array(
        '#type' => 'fieldset',
        '#title' => t('Update subscription'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );

      $form['update_subscription']['subscription_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Subscription name'),
        '#default_value' => $default_name,
      );

      $form['update_subscription']['subscription_amount'] = array(
        '#type' => 'textfield',
        '#title' => t('Amount @symbol', array(
          '@symbol' => commerce_currency_get_symbol($currency_code),
        )),
        '#size' => 10,
        '#default_value' => $default_amount,
      );

      $form['update_subscription']['subscription_submit'] = array(
        '#type' => 'submit',
        '#value' => t('Update subscription details at GoCardless'),
        '#validate' => array('commerce_gc_client_subscription_update_validate'),
        '#submit' => array('commerce_gc_client_subscription_update_submit'),
      );

      // Cancel subscription button.
      $form['cancel_subscription'] = array(
        '#type' => 'submit',
        '#value' => t('Cancel subscription at GoCardless'),
        '#submit' => array('commerce_gc_client_subscription_cancel_submit'),
        '#attributes' => array('onclick' => 'if (!confirm("Are you sure you want to cancel this subscription?")) {return false;}'),
      );
    }
  }

  if ($gc->type == 'P') {

    // The 'Update Scheduled Payment Creation' section for One-off Payments.
    if ($gc->type == 'P' && $gc->gc_mandate_status != 'canceled') {
      $form['update_payment'] = array(
        '#type' => 'fieldset',
        '#title' => t('Update scheduled payments creation'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#access' => $gc->type == 'P' ? TRUE : FALSE,
      );

      $form['update_payment']['next_payment'] = array(
        '#type' => module_exists('date_popup') ? 'date_popup' : 'date_select',
        '#date_format' => module_exists('date_popup') ? variable_get('date_format_medium') : 'd M Y H:i',
        '#default_value' => $gc->next_payment ? date('Y-m-d H:i', $gc->next_payment) : NULL,
        '#date_year_range' => '0:+1',
        '#datepicker_options' => array('minDate' => 0),
        '#description' => t('Change the date that the next scheduled payment will be created. (This is not the same date that the customer will be charged on.)'),
      );

      $form['update_payment']['interval_length'] = array(
        '#type' => 'textfield',
        '#title' => t('Interval length'),
        '#size' => 3,
        '#maxlength' => 3,
        '#default_value' => $data ? $data['interval_params']['length'] : NULL,
        '#element_validate' => array('element_validate_integer_positive'),
      );

      $form['update_payment']['interval_unit'] = array(
        '#type' => 'select',
        '#title' => t('Interval unit'),
        '#default_value' => $data ? $data['interval_params']['unit'] : NULL,
        '#options' => array(
          'weekly' => t('Week'),
          'monthly' => t('Month'),
          'yearly' => t('Year'),
        ),
        '#empty_option' => t('- select -'),
      );

      $form['update_payment']['amount'] = array(
        '#type' => 'textfield',
        '#title' => t('Amount @symbol', array(
          '@symbol' => commerce_currency_get_symbol($commerce_currency_code),
        )),
        '#size' => 8,
        '#default_value' => number_format((float) $total['amount'] / 100, 2, '.', ''),
        '#element_validate' => array('commerce_gc_client_element_validate_zero_or_positive_number'),
      );

      if (isset($line_item->data['gc']['calculate'])) {
        foreach ($line_item->data['gc']['calculate'] as $calculate_key => $calc_amount) {
          $form['update_payment'][$calculate_key] = array(
            '#type' => 'textfield',
            '#title' => check_plain(ucfirst($calculate_key) . ' ' . commerce_currency_get_symbol($commerce_currency_code)),
            '#size' => 8,
            '#default_value' => number_format((float) $calc_amount / 100, 2, '.', ''),
            '#element_validate' => array('commerce_gc_client_element_validate_zero_or_positive_number'),
          );
        }
      }
      $form['update_payment']['update_payment_button'] = array(
        '#type' => 'submit',
        '#value' => t('Update payment'),
        '#submit' => array('commerce_gc_client_update_payment_submit'),
      );
    }

    // The 'Schedule Adjustments' section for One-off Payments.
    if (isset($_GET['adjust'])) {
      $form['adjust_action_message'] = array(
        '#prefix' => '<div style="padding-top:10px;" id="adjustments"></div>',
        '#type' => 'item',
        '#markup' => '<div class="messages status">' . t('Adjustment has been @action.', array(
          '@action' => $_GET['adjust'],
        )) . '</div>',
      );
    }

    $form['adjust'] = array(
      '#type' => 'fieldset',
      '#title' => t('Schedule adjustments'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => t("Adjust the amount of scheduled payment creations for product."),
    );
    if ($gc->gc_mandate_status == 'canceled' && !isset($adjustment_rows)) {
      $form['adjust']['#access'] = FALSE;
    }

    if (isset($adjustment_rows)) {
      $form['adjust']['adjust_table'] = array(
        '#type' => 'fieldset',
        '#title' => t('Adjustments'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );

      $form['adjust']['adjust_table']['table'] = array(
        '#theme' => 'table',
        '#header' => $adjustment_headers,
        '#rows' => isset($adjustment_rows) ? $adjustment_rows : NULL,
        '#empty' => t('There are no scheduled adjustments for this product yet.'),
      );
    }

    if ($gc->gc_mandate_status != 'canceled') {
      $form['adjust']['adjust_title'] = array(
        '#type' => 'textfield',
        '#size' => 24,
        '#title' => t('Adjustment title'),
      );

      $form['adjust']['adjustment'] = array(
        '#title' => t('Adjustment amount @symbol', array(
          '@symbol' => commerce_currency_get_symbol($currency_code),
        )),
        '#size' => 10,
        '#type' => 'textfield',
        '#description' => t('This will adjust the amount of a payment created with GoCardless.'),
      );

      $form['adjust']['payments'] = array(
        '#type' => 'textfield',
        '#size' => 6,
        '#title' => t('Number of payments'),
        '#default_value' => 1,
        '#element_validate' => array('element_validate_integer_positive'),
      );

      $form['adjust']['starting_radio'] = array(
        '#type' => 'radios',
        '#title' => t('Starting from'),
        '#options' => array(
          0 => t('Next scheduled payment creation day'),
          1 => t('Select another date'),
        ),
        '#default_value' => 0,
      );

      $form['adjust']['starting'] = array(
        '#type' => 'date_popup',
        '#title' => t('Starting from'),
        '#description' => t('The adjustment will begin on the first scheduled billing date after that specified here.'),
        '#date_format' => 'd M Y',
        '#date_year_range' => '0:+1',
        '#datepicker_options' => array('minDate' => 0),
        '#states' => array(
          'visible' => array(
            ':input[name="starting_radio"]' => array('value' => 1),
          ),
        ),
      );

      $form['adjust']['plus'] = array(
        '#type' => 'fieldset',
        '#title' => t('and then'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#description' => t("Additional scheduled adjustment to follow initial adjustment"),
      );

      $form['adjust']['plus']['plus_adjustment'] = array(
        '#title' => t('Adjustment amount @symbol', array(
          '@symbol' => commerce_currency_get_symbol($currency_code),
        )),
        '#size' => 10,
        '#type' => 'textfield',
      );

      $form['adjust']['plus']['plus_payments'] = array(
        '#type' => 'textfield',
        '#size' => 6,
        '#title' => t('Number of payments'),
        '#default_value' => NULL,
      );

      $form['adjust']['adjust_button'] = array(
        '#type' => 'submit',
        '#value' => t('Schedule'),
        '#validate' => array('commerce_gc_client_adjust_validate'),
        '#submit' => array('commerce_gc_client_adjust_submit'),
      );
    }
  }

  // Save some variables to the form for use during submission of various
  // form sections.
  $form['order_number'] = array(
    '#type' => 'value',
    '#value' => $order->order_number,
  );
  $form['gcid'] = array(
    '#type' => 'value',
    '#value' => $gc->gcid,
  );
  $form['mandate'] = array(
    '#type' => 'value',
    '#value' => $mandate,
  );
  $form['line_item_select'] = array(
    '#type' => 'value',
    '#value' => $line_item_select,
  );
  $form['currency_code'] = array(
    '#type' => 'value',
    '#value' => $currency_code,
  );

  return $form;
}

/**
 * Form validation handler.
 *
 * Validate a GoCardless subscription update submission.
 *
 * @see commerce_gc_client_subscription_update_submit()
 */
function commerce_gc_client_subscription_update_validate($form, &$form_state) {
  $amount = $form_state['values']['subscription_amount'];
  $currency_code = $form_state['values']['currency_code'];
  if (!is_numeric($amount)) {
    form_set_error('subscription_amount', t('You must enter a numeric amount value.'));
  }
  elseif ($amount < 1) {
    form_set_error('subscription_amount', t('The minimum payment amount for GoCardless is @amount', array(
      '@amount' => commerce_currency_format(100, $currency_code),
    )));
  }
}

/**
 * Form submit handler.
 *
 * Update a GoCardless subscription.
 *
 * @see commerce_gc_client_subscription_update_validate()
 */
function commerce_gc_client_subscription_update_submit($form, &$form_state) {
  $partner = new CommerceGcClientPartner();
  $result = $partner->api([
    'endpoint' => 'subscriptions',
    'action' => 'update',
    'id' => $form_state['values']['subscription_id'],
    'amount' => $form_state['values']['subscription_amount'] * 100,
    'name' => $form_state['values']['subscription_name'],
  ]
  );
  if ($result->response->status_code == 200) {
    drupal_set_message(t('Subscription was updates successfully.'));
  }
}

/**
 * Form submit handler.
 *
 * Cancels a GoCardless subscription.
 */
function commerce_gc_client_subscription_cancel_submit($form, &$form_state) {
  $partner = new CommerceGcClientPartner();
  $result = $partner->api([
    'endpoint' => 'subscriptions',
    'action' => 'cancel',
    'id' => $form_state['values']['subscription_id'],
  ]
  );
  if ($result->response->status_code == 200) {
    drupal_set_message(t('Subscription was cancelled successfully. Please review any pending payments and cancel if necessary.'));
  }
}

/**
 * Form validation handler.
 *
 * Validate a scheduled adjustment submission.
 *
 * @see commerce_gc_client_adjust_submit()
 */
function commerce_gc_client_adjust_validate($form, &$form_state) {
  $adjustment = $form_state['values']['adjustment'];
  if (!empty($adjustment) && !is_numeric($adjustment)) {
    form_set_error('adjustment', t('Adjustment must be a number.'));
  }
  elseif (!empty($adjustment) && (float) $adjustment == 0) {
    form_set_error('adjustment', t('Adjustment cannot be set to zero.'));
  }

  $plus_adjustment = $form_state['values']['plus_adjustment'];
  if (!empty($plus_adjustment) && !is_numeric($plus_adjustment)) {
    form_set_error('plus_adjustment', t('Adjustment must be a number.'));
  }
  elseif (!empty($plus_adjustment) && (float) $plus_adjustment == 0) {
    form_set_error('plus_adjustment', t('Adjustment cannot be set to zero.'));
  }

  if (empty($form_state['values']['payments'])) {

    $form_state['values']['payments'] = 1;
  }

  if (!empty($plus_adjustment) && empty($form_state['values']['plus_payments'])) {
    $form_state['values']['plus_payments'] = 1;
  }
}

/**
 * Form submit handler.
 *
 * Submit a new scheduled adjustment.
 *
 * @see commerce_gc_client_adjust_validate()
 */
function commerce_gc_client_adjust_submit($form, &$form_state) {
  $line_item_id = $form_state['values']['line_item_id'];
  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item_id);
  $unit_price = $line_item_wrapper->commerce_unit_price->value();
  $interval_params = $line_item_wrapper->value()->data['gc']['interval_params'];

  $query = db_select('commerce_gc_client', 'g');
  $query->join('commerce_gc_client_item', 'i', 'g.gcid=i.gcid');
  $starting = $query
    ->fields('i', array('next_payment'))
    ->condition('line_item_id', $line_item_id)
    ->execute()
    ->fetchField();

  if ($form_state['values']['starting_radio'] != 0) {
    $select_date = strtotime($form_state['values']['starting']);
    while ($starting < $select_date) {
      $string_ = '+' . $interval_params['string'];
      $starting = strtotime($string_, $starting);
    }
  }

  // Create array containing scheduled dates.
  $inserts = array();
  $payments = $form_state['values']['payments'];
  $amount = $form_state['values']['adjustment'] * 100;
  for ($i = 0; $i < $payments; $i++) {
    $string_ = '+' . ($i * $interval_params['length']) . ' ' . str_replace('ly', '', $interval_params['unit']);
    $inserts[] = array(
      'timestamp' => strtotime($string_, $starting),
      'amount' => $amount,
    );
    $ending = strtotime($string_, $starting);
  }

  if (isset($form_state['values']['plus_adjustment'])) {

    // Create array containing additional scheduled dates.
    $plus_amount = $form_state['values']['plus_adjustment'] * 100;
    $plus_starting = strtotime('+' . $interval_params['string'], $ending);
    $plus_payments = (int) $form_state['values']['plus_payments'];
    for ($i = 0; $i < $plus_payments; $i++) {
      $string_ = '+' . ($i * $interval_params['length']) . ' ' . str_replace('ly', '', $interval_params['unit']);
      $inserts[] = array(
        'timestamp' => strtotime($string_, $plus_starting),
        'amount' => $plus_amount,
      );
    }
  }

  // Check validity of new scheduled adjustments.
  foreach ($inserts as $insert) {

    $insert_date = date('d M Y', $insert['timestamp']);

    // Calculate sum of scheduled adjustments for date.
    $query = db_select('commerce_gc_client_item', 'i');
    $query->join('commerce_gc_client_item_schedule', 's', 'i.line_item_id = s.line_item_id');
    $adjustments = $query
      ->fields('s')
      ->condition('s.type', 'adj')
      ->condition('s.status', 1)
      ->condition('s.date', $insert_date)
      ->condition('i.line_item_id', $line_item_id)
      ->execute()
      ->fetchAll();

    $sum = 0;
    foreach ($adjustments as $adjustment) {
      $sum = $sum + unserialize($adjustment->data)['amount'];
    }
    $sum = ($sum + $amount + $unit_price['amount']) / 100;

    if ($sum < 1 && $sum != 0) {
      drupal_set_message(t('The schedule cannot be placed because the sum of adjustments, plus the price of the product, for @date is not zero, and is less than @amount, which is not allowed by GoCardless', array(
        '@date' => $insert_date,
        '@amount' => commerce_currency_format(100, $unit_price['currency_code']),
      )), 'warning');
      return;
    }
  }

  // Add schedules data to database.
  foreach ($inserts as $insert) {

    $insert_date = date('d M Y', $insert['timestamp']);
    db_insert('commerce_gc_client_item_schedule')->fields(array(
      'line_item_id' => $line_item_id,
      'type' => 'adj',
      'date' => $insert_date,
      'timestamp' => strtotime($insert_date),
      'status' => 1,
      'data' => serialize(array(
        'title' => !empty($form_state['values']['adjust_title']) ? $form_state['values']['adjust_title'] : 'Adjustment',
        'amount' => $insert['amount'],
      )),
      'created' => REQUEST_TIME,
    ))->execute();
  }
  $order_number = $form_state['values']['order_number'];
  $path = 'admin/commerce/orders/' . $order_number . '/gocardless';
  $form_state['redirect'] = array($path, array(
    'fragment' => 'adjustments',
    'query' => array('adjust' => 'created'),
  ),
  );
}

/**
 * Form validation handler.
 *
 * Validate a GoCardless payment creation submission.
 *
 * @see commerce_gc_client_create_payment_submit()
 */
function commerce_gc_client_create_payment_validate($form, &$form_state) {
  $mandate = $form_state['values']['mandate'];
  $amount = $form_state['values']['payment_amount'];
  $currency_code = $form_state['values']['currency_code'];
  if (!is_numeric($amount)) {
    form_set_error('payment_amount', t('You must enter a numeric amount value.'));
  }
  elseif ($amount < 1) {
    form_set_error('payment_amount', t('The minimum payment amount for GoCardless is @amount', array(
      '@amount' => commerce_currency_format(100, $currency_code),
    )));
  }

  // Check that specified date is greater than or equal to the
  // next possible charge date.
  if (!is_null($form_state['values']['charge_date'])) {
    if (strtotime($form_state['values']['charge_date']) < strtotime($mandate->next_possible_charge_date)) {
      form_set_error('charge_date', t('The date cannot be before the Next Possible Charge Date.'));
    }
  }
}

/**
 * Form submit handler.
 *
 * Submit a GoCardless payment creation.
 *
 * @see commerce_gc_client_create_payment_validate()
 */
function commerce_gc_client_create_payment_submit($form, &$form_state) {
  $currency_code = $form_state['values']['currency_code'];
  $mandate = $form_state['values']['mandate'];
  $amount = $form_state['values']['payment_amount'];
  $order_number = $form_state['values']['order_number'];
  $line_item_id = $form_state['values']['line_item_id'];
  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item_id);
  $line_item_title = $line_item_wrapper->commerce_product->value()->title;

  $partner = new CommerceGcClientPartner();
  $result = $partner->api([
    'endpoint' => 'payments',
    'action' => 'create',
    'mandate' => $mandate->id,
    'amount' => $amount,
    'currency' => $currency_code,
    'description' => !empty($form_state['values']['payment_title']) ? $form_state['values']['payment_title'] : $line_item_title,
    'charge_date' => $form_state['values']['charge_date'],
    'metadata' => array(
      'line_item_id' => $line_item_id,
    ),
  ]
  );
  if (!isset($result->response)) {
    return;
  }
  elseif ($result->response->status_code == 201) {
    $payment = $result->response->body->payments;
  }

  // Provide a hook so that other modules can respond to payment creation.
  module_invoke_all('commerce_gc_client_manual_payment_created', $payment, $line_item_id);
  drupal_goto('admin/commerce/orders/' . $order_number . '/gocardless', array(
    'fragment' => 'payment_create',
    'query' => array('payment_create' => $payment->amount),
  ));
}

/**
 * Form submission handler.
 *
 * Update the payment details for a line item.
 */
function commerce_gc_client_update_payment_submit($form, &$form_state) {
  $next_payment = strtotime($form_state['values']['next_payment']);
  $line_item_id = $form_state['values']['line_item_id'];
  $title = $form_state['values']['line_item_select'][$line_item_id];
  $interval_length = $form_state['values']['interval_length'];
  $interval_unit = $form_state['values']['interval_unit'];
  $amount = $form_state['values']['amount'] * 100;
  $order_id = $form_state['values']['order_number'];
  $order = commerce_order_load($order_id);
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);

  db_update('commerce_gc_client_item')->fields(array(
    'next_payment' => $next_payment,
  ))->condition('line_item_id', $line_item_id, '=')->execute();

  $line_item = commerce_line_item_load($line_item_id);
  $interval_params = array(
    'length' => $interval_length,
    'unit' => $interval_unit,
    'string' => $interval_length . ' ' . str_replace("ly", "", $interval_unit),
    'gc' => $interval_length . ' ' . $interval_unit,
  );
  $line_item->data['gc']['interval_params'] = $interval_params;

  // Look for 'calc' values in the line item's data array (e.g., 'shipping',
  // 'tax', etc.) and rebase the price of line items of a corresponding type,
  // based on the form_state values.
  if (isset($line_item->data['gc']['calculate'])) {
    foreach ($line_item->data['gc']['calculate'] as $calculate_key => $calc_amount) {

      $calculate_new = $form_state['values'][$calculate_key] * 100;
      $calculate_old = $line_item->data['gc']['calculate'][$calculate_key];
      $line_item->data['gc']['calculate'][$calculate_key] = $calculate_new;

      foreach ($order_wrapper->commerce_line_items as $calculate_line_item_wrapper) {
        if ($calculate_line_item_wrapper->type->value() == $calculate_key) {
          $calculate_line_item = $calculate_line_item_wrapper->value();
          $calculate_difference = $calculate_new - $calculate_old;
          $calculate_unit_price_amount = $calculate_line_item_wrapper->commerce_unit_price->amount->value();
          $calculate_line_item_wrapper->commerce_unit_price->amount = $calculate_unit_price_amount + ($calculate_difference * $calculate_line_item->quantity);
          commerce_line_item_rebase_unit_price($calculate_line_item_wrapper->value());
          commerce_line_item_save($calculate_line_item);
        }
      }
    }
  }

  // Rebase the unit price of the line item.
  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
  $line_item_wrapper->commerce_unit_price->amount = $amount / $line_item->quantity;
  commerce_line_item_rebase_unit_price($line_item_wrapper->value());
  commerce_line_item_save($line_item);

  // Update the order total price.
  $order_total = $order_wrapper->commerce_order_total->value();
  $order_total['data']['components'] = array();
  $order_total['data'] = commerce_price_component_add($order_total, 'base_price', $order_total, TRUE);
  $order_wrapper->commerce_order_total = $order_total;
  commerce_order_save($order);

  drupal_set_message(t('Payment creation for @title has been updated.', array(
    '@title' => $title,
  )));
}

/**
 * Page callback: Provides a confirmation form for cancelling payments.
 */
function commerce_gc_client_payment_cancel($form, &$form_state, $order_id, $payment_id) {
  $form['order_id'] = array(
    '#type' => 'value',
    '#value' => $order_id,
  );
  $form['payment_id'] = array(
    '#type' => 'value',
    '#value' => $payment_id,
  );
  $path = array(
    'path' => 'admin/commerce/orders/' . $order_id . '/gocardless',
    'fragment' => 'payment',
    'query' => array('payment' => 'unchanged',
    ),
  );
  return confirm_form($form, t('Are you sure you want to cancel the payment?'), $path, NULL, t('Cancel payment'), t('Go back'));
}

/**
 * Form submission handler for commerce_gc_client_payment_cancel().
 *
 * Submit a GoCardless payment cancellation.
 */
function commerce_gc_client_payment_cancel_submit($form, &$form_state) {
  $order_id = $form_state['values']['order_id'];
  $payment_id = $form_state['values']['payment_id'];
  $partner = new CommerceGcClientPartner();
  $result = $partner->api([
    'endpoint' => 'payments',
    'action' => 'cancel',
    'id' => $payment_id,
  ]
  );
  if ($result->response->status_code == 200) {
    $query_result = 'cancelled';
  }
  else {
    $query_result = 'failed';
  }

  $path = 'admin/commerce/orders/' . $order_id . '/gocardless';
  $form_state['redirect'] = array($path, array(
    'fragment' => 'payment',
    'query' => array(
      'payment' => $query_result,
      'payment_id' => $payment_id,
    ),
  ),
  );
}

/**
 * AJAX callback in response to a new order item being selected for the form.
 *
 * Stores the line item ID as a session variable then refreshes the page.
 *
 * @see commerce_gc_client_mandates_form()
 */
function commerce_gc_client_mandates_form_callback(&$form, &$form_state) {
  global $user;
  $order_number = $form_state['values']['order_number'];
  $_SESSION['commerce_gc_client_line_item_id_' . $user->uid . '_' . $order_number] = $form['line_item_id']['#value'];
  ctools_include('ajax');
  ctools_add_js('ajax-responder');
  $commands[] = ctools_ajax_command_reload();
  print ajax_render($commands);
  drupal_exit();
}

/**
 * Menu callback; confirmation form for scheduled adjustment actions.
 *
 * @param string $action
 *   The action being performed on the scheduled adjustment, either 'cancel'
 *   or  'delete'.
 * @param int $order_id
 *   The Commerce Order ID for the scheduled adjustment.
 * @param int $sid
 *   The ID for the scheduled adjustment.
 *
 * @see commerce_gc_client_scheduled_action_submit()
 */
function commerce_gc_client_scheduled_action($form, &$form_state, $action, $order_id, $sid) {
  $form = array();
  $form['action'] = array(
    '#type' => 'value',
    '#value' => $action,
  );
  $form['order_id'] = array(
    '#type' => 'value',
    '#value' => $order_id,
  );
  $form['sid'] = array(
    '#type' => 'value',
    '#value' => $sid,
  );

  $path = array(
    'path' => 'admin/commerce/orders/' . $order_id . '/gocardless',
    'fragment' => 'adjustments',
  );
  return confirm_form($form, t('Are you sure you want to %action the scheduled adjustment?', array(
    '%action' => $action,
  )), $path, FALSE, t('@action scheduled adjustment', array('@action' => ucfirst($action))), t('Cancel'));
}

/**
 * Form submission handler for commerce_gc_client_scheduled_action().
 */
function commerce_gc_client_scheduled_action_submit($form, &$form_state) {
  $action = $form_state['values']['action'];
  $order_id = $form_state['values']['order_id'];
  $sid = $form_state['values']['sid'];

  if ($action == 'cancel') {
    db_update('commerce_gc_client_item_schedule')->fields(array(
      'status' => 0,
    ))->condition('sid', $sid)->execute();
  }
  elseif ($action == 'delete') {
    db_delete('commerce_gc_client_item_schedule')->condition('sid', $sid)->execute();
  }

  $action == 'cancel' ? $action = 'cancelled' : $action = 'deleted';
  $path = 'admin/commerce/orders/' . $order_id . '/gocardless';
  $form_state['redirect'] = array($path, array(
    'fragment' => 'adjustments',
    'query' => array('adjust' => $action),
  ),
  );
}
