<?php

/**
 * @file
 * Functions for connecting with the GoCardless partner: Seamless-CMS.co.uk.
 */

/**
 * The CommerceGcClientPartner object.
 */
class CommerceGcClientPartner {

  /**
   * Settings for the payment method plus additional partner settings.
   *
   * @var array
   */
  public $settings;

  /**
   * Creates a new instance of CommerceGcClientPartner.
   */
  public function __construct() {
    $this->settings = $this->getSettings();
  }

  /**
   * Fetches the settings.
   *
   * Includes the standard settings for the payment method, plus additional
   * settings required to connect with the GoCardless parter, depending whether
   * the payment method is in live or sandbox mode.
   *
   * @return array
   *   The settings.
   */
  public static function getSettings() {
    $settings = commerce_gc_client_load_settings();
    $sandbox = $settings['connect']['sandbox'];
    $sandbox ? $ext = '_sandbox' : $ext = '_live';
    $settings['partner_user'] = variable_get('commerce_gc_client_user_name' . $ext);
    $settings['partner_pass'] = variable_get('commerce_gc_client_user_pass' . $ext);
    $settings['org_id'] = variable_get('commerce_gc_client_org_id' . $ext);
    $settings['webhook'] = $settings['connect']['webhook' . $ext];
    $settings['partner_url'] = variable_get('commerce_gc_client_partner_url');
    return $settings;
  }

  /**
   * Handles session authentication with GC Partner site.
   *
   * @return int
   *   Returns HTTP response code 200 if authentication is successful, and 401
   *   (Unauthorised) to indicate there is a problem.
   */
  protected function authenticate() {

    if (isset($_SESSION['commerce_gc_client_cookie_created']) && $_SESSION['commerce_gc_client_cookie_created'] < REQUEST_TIME - 1800) {
      unset($_SESSION['commerce_gc_client_cookie']);
      unset($_SESSION['commerce_gc_client_cookie_created']);
    }

    if (!isset($_SESSION['commerce_gc_client_cookie_created'])) {

      // Login first.
      $data = array(
        'username' => $this->settings['partner_user'],
        'password' => $this->settings['partner_pass'],
      );
      $data = drupal_json_encode($data);
      $url = $this->settings['partner_url'] . '/gc_connect/user/login';
      $options = array(
        'headers' => array(
          'Content-Type' => 'application/json',
        ),
        'method' => 'POST',
        'data' => $data,
      );
      $result = drupal_http_request($url, $options);
      $result_data = json_decode($result->data);
      if ($result->code == 200) {

        // Get X-CRSF token, and save cookie and token.
        $_SESSION['commerce_gc_client_cookie'] = $result_data->session_name . '=' . $result_data->sessid;
        $_SESSION['commerce_gc_client_cookie_created'] = REQUEST_TIME;
        $xcrf_url = $this->settings['partner_url'] . '/services/session/token';
        $xcrf_options = array(
          'method' => 'GET',
          'headers' => array(
            'Cookie' => $_SESSION['commerce_gc_client_cookie'],
          ),
        );
        $xcrf_result = drupal_http_request($xcrf_url, $xcrf_options);
        $token = $xcrf_result->data;
        $_SESSION['commerce_gc_client_token'] = $token;
      }
      return $result->code;
    }
    else {
      // Already logged in.
      return $result = 200;
    }
  }

  /**
   * Handles API requests to GC Partner site, and optionally logs results.
   *
   * @params array $params
   *   Array of parameters passed to the GoCardless API.
   *
   * @return mixed
   *   - If $params argument is empty, or there is a problem connecting with
   *     the partner site, then the authentication response code is returned;
   *   - If connection with the GoCardless API is successful, then the object
   *     response of the API request is returned.
   *
   * @see post()
   */
  public function api($params) {
    $authenticate = $this->authenticate();
    if (empty($params)) {
      return $authenticate;
    }
    if ($authenticate == 200) {
      $result = $this->post($params);

      if (isset($result->error)) {
        $message = t('Error calling GoCardless API, code @code (@error)', array(
          '@code' => $result->code,
          '@error' => $result->error,
        ));
        drupal_set_message($message, 'error');
        if ($this->settings['log']['log_api']) {
          watchdog("commerce_gc_client", '@message:<br /><pre>@result</pre>', array(
            '@result' => print_r($result, TRUE),
            '@message' => $message,
          ), WATCHDOG_ERROR);
        }
        $result->response = new stdClass();
        $result->response->status_code = $result->code;
        return $result;
      }
      else {
        if ($this->settings['log']['log_api']) {
          watchdog("commerce_gc_client", 'GoCardless API response: <br /><pre>@result</pre>', array(
            '@result' => print_r($result, TRUE),
          ));
        }
        return $result;
      }
    }
    else {
      drupal_set_message(t('Error @code connecting with partner site', array(
        '@code' => $authenticate,
      )), 'error');
      if (variable_get('commerce_gc_client_debug_api', FALSE)) {
        watchdog("commerce_gc_client", '<pre>@auth</pre>', array(
          '@auth' => print_r($authenticate, TRUE),
        ), WATCHDOG_ERROR);
      }
      return $authenticate;
    }
  }

  /**
   * Posts HTTP requests to GC Partner site.
   *
   * @params array $data
   *   Array of parameters passed to the GoCardless API.
   *
   * @return mixed
   *   Object response to the HTTP request if successful, or string containing
   *   response code and message if failure.
   *
   * @see api()
   */
  private function post($data) {
    $options['headers'] = array(
      'Content-Type' => 'application/json',
      'Cookie' => $_SESSION['commerce_gc_client_cookie'],
      'X-CSRF-Token' => $_SESSION['commerce_gc_client_token'],
    );
    $this->settings['connect']['sandbox'] ? $data['environment'] = 'SANDBOX' : $data['environment'] = 'LIVE';
    $data = drupal_json_encode($data);
    $options['data'] = $data;
    $options['method'] = 'POST';
    $url = $this->settings['partner_url'] . '/gc/client/' . $this->settings['org_id'];
    $response = drupal_http_request($url, $options);

    if (isset($response->error)) {
      return $response;
    }
    else {
      $data = json_decode($response->data);
      return $data;
    }
  }

}
