<?php

/**
 * @file
 * Rules integration for module.
 */

/**
 * Implements hook_rules_action_info().
 */
function commerce_gc_client_rules_action_info() {
  $actions = array();
  $actions['commerce_gc_client_rules_add_to_cart_prevent'] = array(
    'label' => t('Prevent GoCardless product from being added to cart.'),
    'group' => t('Commerce Cart'),
  );
  $actions['commerce_gc_client_rules_calculate_shipping_line_item_data'] = array(
    'label' => t('Add shipping amount to line items GoCardless data.'),
    'group' => t('Commerce Line Item'),
    'parameter' => array(
      'commerce_line_item' => array(
        'type' => 'commerce_line_item',
        'label' => t('Line item'),
      ),
      'amount' => array(
        'type' => 'decimal',
        'label' => t('Amount'),
      ),
    ),
  );
  return $actions;
}

/**
 * Implements hook_rules_condition_info().
 */
function commerce_gc_client_rules_condition_info() {
  $conditions = array();
  $conditions['commerce_gc_client_rules_products_all_gc'] = array(
    'label' => t('All products in cart use GoCardless'),
    'parameter' => array(
      'commerce_order' => array(
        'type' => 'commerce_order',
        'label' => t('Order'),
      ),
      'commerce_product' => array(
        'type' => 'commerce_product',
        'label' => t('Product'),
      ),
    ),
    'group' => t('Commerce Cart'),
  );
  $conditions['commerce_gc_client_rules_products_same_currency'] = array(
    'label' => t('All GoCardless products in cart have same currency'),
    'parameter' => array(
      'commerce_order' => array(
        'type' => 'commerce_order',
        'label' => t('Order'),
      ),
      'commerce_product' => array(
        'type' => 'commerce_product',
        'label' => t('Product'),
      ),
    ),
    'group' => t('Commerce Cart'),
  );
  return $conditions;
}

/**
 * Rules action: Adds a calculated shipping amount to the line_item data.
 *
 * @param object $line_item
 *   Commerce line item entity.
 * @param float $amount
 *   The calculated shipping amount.
 */
function commerce_gc_client_rules_calculate_shipping_line_item_data($line_item, $amount) {
  if ($line_item->type == 'product') {
    $line_item->data['gc']['calculate']['shipping'] = $amount;
    commerce_line_item_save($line_item);
  }
}

/**
 * Rules action: Prevent item from being added to cart.
 *
 * Returns user to the referring page, rather than adding an item to the cart.
 */
function commerce_gc_client_rules_add_to_cart_prevent() {
  drupal_goto($_SERVER['HTTP_REFERER']);
}

/**
 * Rules condition: checks if GoCardless products in cart have same currency.
 *
 * @param object $order
 *   Commerce order entity.
 * @param object $product
 *   Commerce product entity.
 */
function commerce_gc_client_rules_products_same_currency($order, $product) {
  $wrapper = entity_metadata_wrapper('commerce_product', $product);
  $currency_code = $wrapper->commerce_price->value()['currency_code'];

  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  foreach ($order_wrapper->commerce_line_items as $line_item_wrapper) {
    // Ensure that if a cart line item is using GoCardless, then all cart items
    // are of the same currency and the currency is usable by GoCardless.
    $line_item_currency_code = $line_item_wrapper->commerce_unit_price->value()['currency_code'];
    if ($line_item_currency_code != $currency_code) {
      drupal_set_message(t("It is not possible to order direct debit products with different currencies at the same time."), 'warning');
      return TRUE;
    }
  }
  return FALSE;
}

/**
 * Rules condition: check if all products in cart use GoCardless.
 *
 * @param object $order
 *   Commerce order entity.
 * @param object $product
 *   Commerce product entity.
 */
function commerce_gc_client_rules_products_all_gc($order, $product) {
  if (isset($product->data['gc'])) {

    $product->data['gc']['gc_use'] ? $is_gc = TRUE : $is_gc = FALSE;
  }
  else {
    $is_gc = FALSE;
  }

  $wrapper = entity_metadata_wrapper('commerce_order', $order);
  foreach ($wrapper->commerce_line_items as $line_item_wrapper) {
    // Ensure that if a cart line item is using GoCardless, then all cart
    // items are using GoCardless.
    $line_item_data = $line_item_wrapper->value()->data;
    if (isset($line_item_data['gc'])) {
      $line_item_data['gc']['gc_use'] ? $is_also_gc = TRUE : $is_also_gc = FALSE;
    }
    else {
      $is_also_gc = FALSE;
    }

    if (($is_gc && !$is_also_gc) || (!$is_gc && $is_also_gc)) {
      drupal_set_message(t("It is not possible to order direct debit products at the same time as non-direct debit products."), 'warning');
      return TRUE;
    }
  }
  return FALSE;
}
